@@ -81,6 +81,10 @@ extern "C" {
 #include <Lum/TextValue.h>
 #include <Lum/View.h>
 
+#include "Configuration.h"
+#include "Settings.h"
+
+#include "GPSData.h"
 #include "Compass.h"
 #include "Speed.h"
 #include "TimeLine.h"
@@ -88,18 +92,6 @@ extern "C" {
 
 #include <iostream>
 
-// maximum age of data in seconds until it is invalid
-#if defined(HAVE_LIB_GPS)
-// Using normal libgps we don't trust situation if GPS daemon is not
-// responding in time
-static long obsoleteDataAge = 30;
-#else
-// Using liblocation on the other hand we trust it to send diffs
-// every time something relvant changes.
-// Data does not age!
-static long obsoleteDataAge = std::numeric_limits<long>::max();
-#endif
-
 static long reconnectTime   = 10; // Period of time after which we check for disconnection and
                                   // trigger reconnection
 static long refreshTime     = 10; // Period of time after which we update the stream views
@@ -142,391 +134,6 @@ std::wstring SystemTimeToISO8601(const Lum::Base::SystemTime& time)
   return res;
 }
 
-class GPSData
-{
-public:
-  struct Satellite
-  {
-    int  id;
-    bool used;
-    int  snr;
-  };
-
-private:
-  bool                   connected;
-  Lum::Base::SystemTime  connectedLastData;
-
-  bool                   online;
-  Lum::Base::SystemTime  onlineLastData;
-
-  bool                   fix;
-  Lum::Base::SystemTime  fixLastData;
-
-  double                 latitude;
-  double                 latitudeDev;
-  Lum::Base::SystemTime  latitudeLastData;
-
-  double                 longitude;
-  double                 longitudeDev;
-  Lum::Base::SystemTime  longitudeLastData;
-
-  double                 altitude;
-  double                 altitudeDev;
-  Lum::Base::SystemTime  altitudeLastData;
-
-  double                 speed;
-  double                 speedDev;
-  Lum::Base::SystemTime  speedLastData;
-
-  double                 track;
-  double                 trackDev;
-  Lum::Base::SystemTime  trackLastData;
-
-  double                 climb;
-  double                 climbDev;
-  Lum::Base::SystemTime  climbLastData;
-
-  Lum::Base::SystemTime  time;
-  double                 timeDev;
-  Lum::Base::SystemTime  timeLastData;
-
-  std::vector<Satellite> satellites;
-  Lum::Base::SystemTime  satellitesLastData;
-
-public:
-  GPSData()
-   : connectedLastData(0),
-     onlineLastData(0),
-     fixLastData(0),
-     latitudeLastData(0),
-     longitudeLastData(0),
-     altitudeLastData(0),
-     speedLastData(0),
-     trackLastData(0),
-     climbLastData(0),
-     timeLastData(0),
-     satellitesLastData(0)
-  {
-    // no code
-  }
-
-  //
-  // Connected
-  //
-  void SetConnected(bool connected)
-  {
-    connectedLastData.SetToNow();
-    this->connected=connected;
-  }
-
-  bool HasConnected() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-connectedLastData<obsoleteDataAge;
-  }
-
-  bool GetConnected() const
-  {
-    assert(HasConnected());
-
-    return connected;
-  }
-
-  //
-  // Online
-  //
-  void SetOnline(bool online)
-  {
-    onlineLastData.SetToNow();
-    this->online=online;
-  }
-
-  bool HasOnline() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-onlineLastData<obsoleteDataAge;
-  }
-
-  bool GetOnline() const
-  {
-    assert(HasOnline());
-
-    return online;
-  }
-
-  //
-  // Fix
-  //
-  void SetFix(bool fix)
-  {
-    fixLastData.SetToNow();
-    this->fix=fix;
-  }
-
-  bool HasFix() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-fixLastData<obsoleteDataAge;
-  }
-
-  bool GetFix() const
-  {
-    assert(HasFix());
-
-    return fix;
-  }
-
-  //
-  // Latitude
-  //
-  void SetLatitude(double latitude,
-                   double latitudeDev)
-  {
-    latitudeLastData.SetToNow();
-    this->latitude=latitude;
-    this->latitudeDev=latitudeDev;
-  }
-
-  bool HasLatitude() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-latitudeLastData<obsoleteDataAge;
-  }
-
-  double GetLatitude() const
-  {
-    assert(HasLatitude());
-
-    return latitude;
-  }
-
-  double GetLatitudeDev() const
-  {
-    assert(HasLatitude());
-
-    return latitudeDev;
-  }
-
-  //
-  // Longitude
-  //
-  void SetLongitude(double longitude,
-                    double longitudeDev)
-  {
-    longitudeLastData.SetToNow();
-    this->longitude=longitude;
-    this->longitudeDev=longitudeDev;
-  }
-
-  bool HasLongitude() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-longitudeLastData<obsoleteDataAge;
-  }
-
-  double GetLongitude() const
-  {
-    assert(HasLongitude());
-
-    return longitude;
-  }
-
-  double GetLongitudeDev() const
-  {
-    assert(HasLongitude());
-
-    return longitudeDev;
-  }
-
-  //
-  // Altitude
-  //
-  void SetAltitude(double altitude,
-                   double altitudeDev)
-  {
-    altitudeLastData.SetToNow();
-    this->altitude=altitude;
-    this->altitudeDev=altitudeDev;
-  }
-
-  bool HasAltitude() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-altitudeLastData<obsoleteDataAge;
-  }
-
-  double GetAltitude() const
-  {
-    assert(HasAltitude());
-
-    return altitude;
-  }
-
-  double GetAltitudeDev() const
-  {
-    assert(HasAltitude());
-
-    return altitudeDev;
-  }
-
-  //
-  // Speed
-  //
-  void SetSpeed(double speed,
-                double speedDev)
-  {
-    speedLastData.SetToNow();
-    this->speed=speed;
-    this->speedDev=speedDev;
-  }
-
-  bool HasSpeed() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-speedLastData<obsoleteDataAge;
-  }
-
-  double GetSpeed() const
-  {
-    assert(HasSpeed());
-
-    return speed;
-  }
-
-  double GetSpeedDev() const
-  {
-    assert(HasSpeed());
-
-    return speedDev;
-  }
-
-  //
-  // Track
-  //
-  void SetTrack(double track,
-                double trackDev)
-  {
-    trackLastData.SetToNow();
-    this->track=track;
-    this->trackDev=trackDev;
-  }
-
-  bool HasTrack() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-trackLastData<obsoleteDataAge;
-  }
-
-  double GetTrack() const
-  {
-    assert(HasTrack());
-
-    return track;
-  }
-
-  double GetTrackDev() const
-  {
-    assert(HasTrack());
-
-    return trackDev;
-  }
-
-  //
-  // Climb
-  //
-  void SetClimb(double climb,
-                double climbDev)
-  {
-    climbLastData.SetToNow();
-    this->climb=climb;
-    this->climbDev=climbDev;
-  }
-
-  bool HasClimb() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-climbLastData<obsoleteDataAge;
-  }
-
-  double GetClimb() const
-  {
-    assert(HasClimb());
-
-    return climb;
-  }
-
-  double GetClimbDev() const
-  {
-    assert(HasClimb());
-
-    return climbDev;
-  }
-
-  //
-  // Time
-  //
-  void SetTime(Lum::Base::SystemTime& time,
-               double timeDev)
-  {
-    timeLastData.SetToNow();
-    this->time=time;
-    this->timeDev=timeDev;
-  }
-
-  bool HasTime() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-timeLastData<obsoleteDataAge;
-  }
-
-  Lum::Base::SystemTime GetTime() const
-  {
-    assert(HasTime());
-
-    return time;
-  }
-
-  double GetTimeDev() const
-  {
-    assert(HasTime());
-
-    return timeDev;
-  }
-
-  //
-  // Satellites
-  //
-  void SetSatellites(const std::vector<Satellite>& satellites)
-  {
-    satellitesLastData.SetToNow();
-    this->satellites=satellites;
-  }
-
-  bool HasSatellites() const
-  {
-    Lum::Base::SystemTime now;
-
-    return now-satellitesLastData<obsoleteDataAge;
-  }
-
-  const std::vector<Satellite>& GetSatellites() const
-  {
-    assert(HasSatellites());
-
-    return satellites;
-  }
-};
-
 typedef Lum::Model::StdTable<GPSData::Satellite> SatellitesModel;
 typedef Lum::Base::Reference<SatellitesModel>    SatellitesModelRef;
 
@@ -873,6 +480,7 @@ private:
   Lum::Model::ActionRef    stopTrackingAction;
   Lum::Model::ActionRef    deleteTrackAction;
   Lum::Model::ActionRef    exportTrackAction;
+  Lum::Model::ActionRef    settingsAction;
   Lum::Model::ActionRef    aboutAction;
 
   Lum::Model::ActionRef    dataChangedAction;
@@ -880,7 +488,6 @@ private:
   Lum::Model::ActionRef    reconnectCheckTimer;
 
   Lum::Model::BooleanRef   tracking;
-  Lum::Model::BooleanRef   prohibitScreenBlanking;
 
   std::ofstream            trackFile;
 
@@ -1067,12 +674,12 @@ public:
     stopTrackingAction(new Lum::Model::Action()),
     deleteTrackAction(new Lum::Model::Action()),
     exportTrackAction(new Lum::Model::Action()),
+    settingsAction(new Lum::Model::Action()),
     aboutAction(new Lum::Model::Action()),
     dataChangedAction(new Lum::Model::Action()),
     refreshTimer(new Lum::Model::Action()),
     reconnectCheckTimer(new Lum::Model::Action()),
-    tracking(new Lum::Model::Boolean(false)),
-    prohibitScreenBlanking(new Lum::Model::Boolean(false))
+    tracking(new Lum::Model::Boolean(false))
 #if defined(HAVE_LIB_LOCATION)
     ,device(NULL)
     ,control(NULL)
@@ -1094,6 +701,7 @@ public:
     Observe(stopTrackingAction);
     Observe(deleteTrackAction);
     Observe(exportTrackAction);
+    Observe(settingsAction);
     Observe(aboutAction);
 
     Observe(dataChangedAction);
@@ -1101,10 +709,7 @@ public:
     Observe(reconnectCheckTimer);
     Observe(trackSelection);
 
-    if (Lum::OS::Manager::repository->GetDisplayManager()==NULL) {
-      prohibitScreenBlanking->Disable();
-    }
-
+    Observe(unitMode);
     Observe(prohibitScreenBlanking);
 
     ::window=this;
@@ -1245,8 +850,7 @@ public:
         ->ActionQuit(GetClosedAction())
       ->End()
       ->GroupEdit()
-        ->Boolean(Lum::Def::Boolean(Lum::Def::Desc(L"Prohibit screen blanking"),
-                                    prohibitScreenBlanking))
+        ->ActionSettings(settingsAction)
       ->End();
 
     Lum::Def::Menu *postMenu=Lum::Def::Menu::Create();
@@ -1837,7 +1441,7 @@ public:
 
       if (data.GetLatitudeDev()!=0 &&
           finite(data.GetLatitudeDev())) {
-        l+=std::wstring(L"\u00b1")+DoubleToString(data.GetLatitudeDev())+L"m";
+        l+=std::wstring(L"\u00b1")+GetMetricAndUnitAsString(data.GetLatitudeDev());
       }
 
       latitude->Set(l);
@@ -1853,7 +1457,7 @@ public:
 
       if (data.GetLongitudeDev()!=0 &&
           finite(data.GetLongitudeDev())) {
-        l+=std::wstring(L"\u00b1")+DoubleToString(data.GetLongitudeDev())+L"m";
+        l+=std::wstring(L"\u00b1")+GetMetricAndUnitAsString(data.GetLongitudeDev());
       }
 
       longitude->Set(l);
@@ -1863,47 +1467,23 @@ public:
     }
 
     if (data.HasAltitude()) {
-      std::wstring a=DoubleToString(data.GetAltitude());
-
-      if (data.GetAltitudeDev()!=0 &&
-          finite(data.GetAltitudeDev())) {
-        a+=L"\u00b1"+DoubleToString(data.GetAltitudeDev());
-      }
-
-      a+=L"m";
-
-      altitude->Set(a);
+      altitude->Set(GetMetricAndUnitAsString(data.GetAltitude(),
+                                             data.GetAltitudeDev()));
     }
     else {
       altitude->SetNull();
     }
 
     if (data.HasSpeed()) {
+      double       speed=data.GetSpeed();
+      double       speedDev=data.GetSpeedDev();
       std::wstring s;
+      
       // We expect m/s here
-      if (data.GetSpeed()>=1/3.6) {
-        s=DoubleToString(data.GetSpeed()*3600.0/1000);
+      s=GetSpeedAndUnitAsString(speed,speedDev);
 
-        if (data.GetSpeedDev()!=0 &&
-            finite(data.GetSpeedDev())) {
-          s+=L"\u00b1"+DoubleToString(data.GetSpeedDev()*3600.0/1000);
-        }
-
-        s+=L"km/h";
-      }
-      else {
-        s=DoubleToString(data.GetSpeed());
-
-        if (data.GetSpeedDev()!=0 &&
-            finite(data.GetSpeedDev())) {
-          s+=L"\u00b1"+DoubleToString(data.GetSpeedDev());
-        }
-
-        s+=L"m/s";
-      }
-
-      speedValue->Set(data.GetSpeed());
-      speed->Set(s);
+      speedValue->Set(speed);
+      this->speed->Set(s);
     }
     else {
       speedValue->SetNull();
@@ -1930,16 +1510,8 @@ public:
     }
 
     if (data.HasClimb()) {
-      std::wstring c=DoubleToString(data.GetClimb());
-
-      if (data.GetClimbDev()!=0 &&
-          finite(data.GetClimbDev())) {
-        c+=L"\u00b1"+DoubleToString(data.GetClimbDev());
-      }
-
-      c+=L"m";
-
-      climb->Set(c);
+      climb->Set(GetMetricAndUnitAsString(data.GetClimb(),
+                                          data.GetClimbDev()));
     }
     else {
       climb->SetNull();
@@ -2013,6 +1585,21 @@ public:
     }
   }
 
+  void ShowSettings()
+  {
+    Settings* settings;
+
+    settings=new Settings();
+    settings->SetParent(this);
+    if (settings->Open()) {
+      settings->SetExitAction(settings->GetClosedAction());
+      settings->EventLoop();
+      settings->Close();
+    }
+
+    delete settings;
+  }
+
   void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
   {
     if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
@@ -2020,9 +1607,11 @@ public:
       Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
       RefreshTracks();
+      LoadConfig();
     }
     else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
       DeinitializeGPS();
+      SaveConfig();
     }
     else if (model==aboutAction && aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
@@ -2079,6 +1668,9 @@ public:
         exportTrackAction->Disable();
       }
     }
+    else if (model==settingsAction && settingsAction->IsFinished()) {
+      ShowSettings();
+    }
     else if (model==prohibitScreenBlanking) {
       Lum::OS::Manager::DisplayManager *displayManager=Lum::OS::Manager::repository->GetDisplayManager();
 
@@ -2086,6 +1678,14 @@ public:
         displayManager->AllowScreenBlanking(!prohibitScreenBlanking->Get());
       }
     }
+    else if (model==unitMode) {
+      std::cout << "unitMode changed!" << std::endl;
+      GPSData dataCopy;
+
+      GetGPSDataCopy(dataCopy);
+      ShowChange(dataCopy);
+      //UpdateStreams(dataCopy);
+    }
 
     Dialog::Resync(model,msg);
   }
