@@ -21,6 +21,7 @@
 
 #include <Lum/OS/Font.h>
 
+#include "Configuration.h"
 #include "Util.h"
 
 Speed::Speed()
@@ -65,17 +66,25 @@ void Speed::Draw(int x, int, size_t w, size_t h)
   Lum::OS::FontRef   bigFont=Lum::OS::display->GetFont(750);
   size_t             valueWidth;
 
-  if (!model.Valid() || model->IsNull()) {
-    value=L"-,--";
-    unit=L"m/s";
-  }
-  else if (model->Get()>=1/3.6) {
-    value=DoubleToString(model->Get()*3600.0/1000);
-    unit=L"km/h";
-  }
+  if (unitMode->Get()==1) {
+    if (!model.Valid() || model->IsNull()) {
+        value=L"-,--";
+        unit=GetSpeedNoDataUnit();
+    }
+    else  {
+      value=GetSpeedAsString(model->Get());
+      unit=GetSpeedUnitAsString(model->Get());
+    }
+  }  
   else {
-    value=DoubleToString(model->Get());
-    unit=L"m/s";
+    if (!model.Valid() || model->IsNull()) {
+     value=L"-,--";
+     unit=GetSpeedNoDataUnit();
+    }
+    else {
+      value=GetSpeedAsString(model->Get());
+      unit=GetSpeedUnitAsString(model->Get());
+    }
   }
 
   draw->PushForeground(Lum::OS::display->GetColor(Lum::OS::Display::textColor));
