@@ -24,5 +24,12 @@
 
 extern std::wstring DoubleToString(double value);
 
+extern std::wstring GetSpeedNoDataUnit();
+extern std::wstring GetSpeedAsString(double metric);
+extern std::wstring GetSpeedUnitAsString(double metric);
+extern std::wstring GetSpeedAndUnitAsString(double metric);
+extern std::wstring GetSpeedAndUnitAsString(double metric, double dev);
 
+extern std::wstring GetMetricAndUnitAsString(double metric);
+extern std::wstring GetMetricAndUnitAsString(double metric, double dev);
 #endif
